@@ -45,7 +45,7 @@ else:
     DEVNULL = subprocess.DEVNULL
 
 # Wrapper version
-VERSION = "19"
+VERSION = "20"
 
 LOG_LEVEL = logging.DEBUG
 STATE_DIR = '/tmp'
@@ -115,7 +115,7 @@ class BaseHost(object):
     def create_runner(self, *args, **kwargs):
         raise NotImplementedError()
 
-    def getLogs(self):
+    def get_logs(self):
         return ('/tmp', '/tmp')
 
     def get_tag(self):
@@ -171,7 +171,7 @@ class CNVHost(BaseHost):
     def create_runner(self, *args, **kwargs):
         return SubprocessRunner(self, *args, **kwargs)
 
-    def getLogs(self):
+    def get_logs(self):
         # TODO: we should either pipe everything to stdout or push to log
         # collector
         return ('/tmp', '/tmp')
@@ -262,18 +262,21 @@ class K8SCommunicator(object):
             self._token = f.read()
 
         self._url = (
-            'https://{host}:%{port}'
+            'https://{host}:{port}'
             '/api/v1/namespaces/{ns}/pods/{pod}').format(
                 host=self._host,
                 port=self._port,
                 ns=self._ns,
                 pod=self._pod)
+        # too early for logging
+        # logging.info('Accessing Kubernetes on: %s', self._url)
         self._headers = [
             'Authorization: Bearer {}'.format(self._token),
             'Accept: application/json',
         ]
 
     def get(self):
+        logging.debug('Accessing Kubernetes on: %s', self._url)
         response = BytesIO()
         c = pycurl.Curl()
         # c.setopt(pycurl.VERBOSE, 1)
@@ -290,6 +293,7 @@ class K8SCommunicator(object):
         return response.getvalue()
 
     def patch(self, body):
+        logging.debug('Accessing Kubernetes on: %s', self._url)
         data = BytesIO(body.encode('utf-8'))
         response = BytesIO()
         c = pycurl.Curl()
@@ -322,7 +326,7 @@ class OSPHost(BaseHost):
     def create_runner(self, *args, **kwargs):
         return SystemdRunner(self, *args, **kwargs)
 
-    def getLogs(self):
+    def get_logs(self):
         log_dir = '/var/log/virt-v2v'
         if not os.path.isdir(log_dir):
             os.makedirs(log_dir)
@@ -651,7 +655,7 @@ class VDSMHost(BaseHost):
     def create_runner(self, *args, **kwargs):
         return SystemdRunner(self, *args, **kwargs)
 
-    def getLogs(self):
+    def get_logs(self):
         """ Returns tuple with directory for virt-v2v log and wrapper log """
         return (self.VDSM_LOG_DIR, self.VDSM_LOG_DIR)
 
@@ -738,6 +742,7 @@ class VDSMHost(BaseHost):
 
     def prepare_command(self, data, v2v_args, v2v_env, v2v_caps):
         v2v_args.extend([
+            '--bridge', 'ovirtmgmt',
             '-of', data['output_format'],
             ])
         if 'allocation' in data:
@@ -946,7 +951,12 @@ class VDSMHost(BaseHost):
 
 
 class State(object):  # {{{
-    """ State object (which is a dict inside) implemented as singleton """
+    """
+    State object (which is a dict inside) implemented as singleton
+
+    This is not just the contain of state file, but it contains all the
+    internal configuration.
+    """
     class __StateObject:
         def __init__(self):
             # For now keep content as dict. Idealy this should be changed
@@ -965,24 +975,35 @@ class State(object):  # {{{
                     'network': None,
                     }
                 }
-            self._filename = None
+            self.state_file = None
+            self.v2v_log = None
+            self.machine_readable_log = None
 
         def __getattr__(self, name):
             return getattr(self._state, name)
 
+        def __getitem__(self, key):
+            return self._state[key]
+
+        def __setitem__(self, key, value):
+            self._state[key] = value
+
         def __str__(self):
             return repr(self._state)
 
-        def get_filename(self):
-            return self._filename
+        # def get_filename(self):
+        #     return self._filename
 
-        def set_filename(self, name):
-            self._filename = name
+        # def set_filename(self, name):
+        #     self._filename = name
+
+        # FIXME: property attributes (property()/@property) don't work properly
+        # filename = property(get_filename, set_filename)
 
         def write(self):
             state = self._state.copy()
             del state['internal']
-            with open(self._filename, 'w') as f:
+            with open(self.state_file, 'w') as f:
                 json.dump(state, f)
 
     instance = None
@@ -1032,7 +1053,8 @@ def error(short_message, *args, **kwargs):
     Used for error reporting, e.g.:
 
         error('Failed create port')
-        error('sock=%r; pid=%r', sock, pid)
+        error('Error starting ssh-agent',
+              'Incomplete match: sock=%r; pid=%r', sock, pid)
         error(e.args[0],
               'An error occured, finishing state file...',
               exception=True)
@@ -1128,18 +1150,34 @@ class OutputParser(object):  # {{{
         br' \'?(?P<uuid>[a-fA-F0-9-]*)\'?$')
     SSH_VMX_GUEST_NAME = re.compile(br'^displayName = "(.*)"$')
 
-    def __init__(self, v2v_log, duplicate=False):
-        # Wait for the log file to appear
+    def __init__(self, duplicate=False):
+        state = State().instance
+        # Wait for the log files to appear
         for i in range(10):
-            if os.path.exists(v2v_log):
+            if os.path.exists(state.v2v_log) \
+                    and os.path.exists(state.machine_readable_log):
                 continue
             time.sleep(1)
-        self._log = open(v2v_log, 'rbU')
+        self._log = open(state.v2v_log, 'rbU')
+        self._machine_log = open(state.machine_readable_log, 'rbU')
         self._current_disk = None
         self._current_path = None
         self._duplicate = duplicate
 
     def parse(self, state):
+        line = self._machine_log.readline()
+        while line != b'':
+            try:
+                message = json.loads(line)
+                if message.get('type') == 'error':
+                    message = message.get('message')
+                    error('virt-v2v error: {}'.format(message))
+            except json.decoder.JSONDecodeError:
+                logging.exception(
+                    'Failed to parse line from'
+                    ' virt-v2v machine readable output')
+                logging.error('Offending line: %r', line)
+            line = self._machine_log.readline()
         line = self._log.readline()
         while line != b'':
             if self._duplicate:
@@ -1172,7 +1210,7 @@ class OutputParser(object):  # {{{
         # VDDK
         m = self.NBDKIT_DISK_PATH_RE.match(line)
         if m is not None:
-            self._current_path = m.group(1)
+            self._current_path = m.group(1).decode('utf-8')
             if self._current_disk is not None:
                 logging.info('Copying path: %s', self._current_path)
                 self._locate_disk(state)
@@ -1190,7 +1228,7 @@ class OutputParser(object):  # {{{
             path = m.group(1)
             # Transform path to be raltive to storage
             self._current_path = self.VMDK_PATH_RE.sub(
-                br'[\g<store>] \g<vm>/\g<disk>.vmdk', path)
+                br'[\g<store>] \g<vm>/\g<disk>.vmdk', path).decode('utf-8')
             if self._current_disk is not None:
                 logging.info('Copying path: %s', self._current_path)
                 self._locate_disk(state)
@@ -1201,7 +1239,7 @@ class OutputParser(object):  # {{{
             path = m.group(1)
             # Transform path to be raltive to storage
             self._current_path = self.VMDK_PATH_RE.sub(
-                br'[\g<store>] \g<vm>/\g<disk>.vmdk', path)
+                br'[\g<store>] \g<vm>/\g<disk>.vmdk', path).decode('utf-8')
             if self._current_disk is not None:
                 logging.info('Copying path: %s', self._current_path)
                 self._locate_disk(state)
@@ -1407,7 +1445,7 @@ class SystemdRunner(BaseRunner):  # {{{
                 )
         run_output = proc.communicate()[0]
         logging.info('systemd-run returned: %s', run_output)
-        m = re.search(br'\b(run-[0-9]+\.service)\.', run_output)
+        m = re.search(br'\b(run-r?[0-9a-f]+\.service)', run_output)
         if m is None:
             raise RuntimeError(
                 'Failed to find service name in output',
@@ -1421,7 +1459,7 @@ class SystemdRunner(BaseRunner):  # {{{
             time.sleep(5)
         if pid is None or pid == '':
             raise RuntimeError('Failed to get PID for virt-v2v process')
-            logging.info('Running with PID: %s', pid)
+        logging.info('Running with PID: %s', pid)
         try:
             self._pid = int(pid)
         except ValueError:
@@ -1441,7 +1479,7 @@ class SystemdRunner(BaseRunner):  # {{{
                 'Failed to get "%s" for virt-v2v service from systemd',
                 property_name)
             return None
-        m = re.match(br'^%s=(.*)$' % property_name, output)
+        m = re.match(br'^%s=(.*)$' % property_name.encode('utf-8'), output)
         if m is not None:
             return m.group(1)
         else:
@@ -1677,15 +1715,15 @@ class TcController(object):
             return None
         # Split into words by line
         output = output.splitlines()
-        output = list(map(str.split, output))
+        output = list(map(bytes.split, output))
         return output
 
 
 @contextmanager
-def log_parser(v2v_log, duplicate=False):
+def log_parser(duplicate=False):
     parser = None
     try:
-        parser = OutputParser(v2v_log, duplicate)
+        parser = OutputParser(duplicate)
         yield parser
     finally:
         if parser is not None:
@@ -1693,11 +1731,12 @@ def log_parser(v2v_log, duplicate=False):
 
 
 def prepare_command(data, v2v_caps, agent_sock=None):
+    state = State().instance
     v2v_args = [
         '-v', '-x',
         data['vm_name'],
-        '--bridge', 'ovirtmgmt',
-        '--root', 'first'
+        '--root', 'first',
+        '--machine-readable=file:{}'.format(state.machine_readable_log),
     ]
 
     if data['transport_method'] == 'vddk':
@@ -1825,8 +1864,9 @@ def throttling_update(runner, initial=None):
     logging.info('New throttling setup: %r', state['throttling'])
 
 
-def wrapper(host, data, state, v2v_log, v2v_caps, agent_sock=None):
+def wrapper(host, data, v2v_caps, agent_sock=None):
 
+    state = State().instance
     v2v_args, v2v_env = prepare_command(data, v2v_caps, agent_sock)
     v2v_args, v2v_env = host.prepare_command(
         data, v2v_args, v2v_env, v2v_caps)
@@ -1834,7 +1874,7 @@ def wrapper(host, data, state, v2v_log, v2v_caps, agent_sock=None):
     logging.info('Starting virt-v2v:')
     log_command_safe(v2v_args, v2v_env)
 
-    runner = host.create_runner(v2v_args, v2v_env, v2v_log)
+    runner = host.create_runner(v2v_args, v2v_env, state.v2v_log)
     try:
         runner.run()
     except RuntimeError as e:
@@ -1849,7 +1889,7 @@ def wrapper(host, data, state, v2v_log, v2v_caps, agent_sock=None):
     try:
         state['started'] = True
         state.write()
-        with log_parser(v2v_log, not data['daemonize']) as parser:
+        with log_parser(not data['daemonize']) as parser:
             while runner.is_running():
                 state = parser.parse(state)
                 state.write()
@@ -1883,29 +1923,40 @@ def write_password(password, password_files, uid, gid):
     return pfile[1]
 
 
-def spawn_ssh_agent(data):
+def spawn_ssh_agent(data, uid, gid):
+    cmd = [
+        'setpriv', '--reuid=%d' % uid, '--regid=%d' % gid, '--clear-groups',
+        'ssh-agent']
+    try:
+        out = subprocess.check_output(
+            cmd,
+            stderr=subprocess.STDOUT,
+            stdin=DEVNULL)
+    except subprocess.CalledProcessError as e:
+        error('Failed to start ssh-agent', exception=True)
+        logging.error('Command failed with: %s', e.output)
+        return None, None
+    logging.debug('ssh-agent: %s' % out)
+    sock = re.search(br'^SSH_AUTH_SOCK=([^;]+);', out, re.MULTILINE)
+    pid = re.search(br'^echo Agent pid ([0-9]+);', out, re.MULTILINE)
+    if not sock or not pid:
+        error(
+            'Error starting ssh-agent',
+            'Incomplete match of ssh-agent output; sock=%r; pid=%r',
+            sock, pid)
+        return None, None
     try:
-        out = subprocess.check_output(['ssh-agent'])
-        logging.debug('ssh-agent: %s' % out)
-        sock = re.search(br'^SSH_AUTH_SOCK=([^;]+);', out, re.MULTILINE)
-        pid = re.search(br'^echo Agent pid ([0-9]+);', out, re.MULTILINE)
-        if not sock or not pid:
-            error(
-                'Incomplete match of ssh-agent output; sock=%r; pid=%r',
-                sock, pid)
-            return None, None
         agent_sock = sock.group(1)
         agent_pid = int(pid.group(1))
-        logging.info('SSH Agent started with PID %d', agent_pid)
-    except subprocess.CalledProcessError:
-        error('Failed to start ssh-agent', exception=True)
-        return None, None
     except ValueError:
         error('Failed to parse ssh-agent output', exception=True)
         return None, None
+    logging.info('SSH Agent started with PID %d', agent_pid)
     env = os.environ.copy()
     env['SSH_AUTH_SOCK'] = agent_sock
-    cmd = ['ssh-add']
+    cmd = [
+        'setpriv', '--reuid=%d' % uid, '--regid=%d' % gid, '--clear-groups',
+        'ssh-add']
     if 'ssh_key_file' in data:
         logging.info('Using custom SSH key')
         cmd.append(data['ssh_key_file'])
@@ -1927,8 +1978,8 @@ def spawn_ssh_agent(data):
 
 def virt_v2v_capabilities():
     try:
-        return subprocess.check_output(['virt-v2v', u'--machine-readable']) \
-                         .split('\n')
+        out = subprocess.check_output(['virt-v2v', u'--machine-readable'])
+        return out.decode('utf-8').split('\n')
     except subprocess.CalledProcessError:
         logging.exception('Failed to start virt-v2v')
         return None
@@ -2029,19 +2080,20 @@ def main():
     if 'daemonize' not in data:
         data['daemonize'] = True
 
+    state = State().instance
     host_type = BaseHost.detect(data)
     host = BaseHost.factory(host_type)
 
     # The logging is delayed after we now which user runs the wrapper.
     # Otherwise we would have two logs.
     log_tag = host.get_tag()
-    log_dirs = host.getLogs()
-    v2v_log = os.path.join(log_dirs[0], 'v2v-import-%s.log' % log_tag)
+    log_dirs = host.get_logs()
+    state.v2v_log = os.path.join(log_dirs[0], 'v2v-import-%s.log' % log_tag)
+    state.machine_readable_log = os.path.join(
+        log_dirs[0], 'v2v-import-%s-mr.log' % log_tag)
     wrapper_log = os.path.join(log_dirs[1],
                                'v2v-import-%s-wrapper.log' % log_tag)
-    state = State().instance
-    state.set_filename(
-        os.path.join(STATE_DIR, 'v2v-import-%s.state' % log_tag))
+    state.state_file = os.path.join(STATE_DIR, 'v2v-import-%s.state' % log_tag)
     throttling_file = os.path.join(STATE_DIR,
                                    'v2v-import-%s.throttle' % log_tag)
     state['internal']['throttling_file'] = throttling_file
@@ -2055,8 +2107,8 @@ def main():
 
     logging.info('Wrapper version %s, uid=%d', VERSION, os.getuid())
 
-    logging.info('Will store virt-v2v log in: %s', v2v_log)
-    logging.info('Will store state file in: %s', state.get_filename())
+    logging.info('Will store virt-v2v log in: %s', state.v2v_log)
+    logging.info('Will store state file in: %s', state.state_file)
     logging.info('Will read throttling limits from: %s', throttling_file)
 
     password_files = []
@@ -2139,11 +2191,8 @@ def main():
                 logging.debug('Initializing disk list from %r',
                               data['source_disks'])
                 for d in data['source_disks']:
-                    # NOTE: We expect names from virt-v2v/VMware to be UTF-8
-                    # encoded. Encoding them here is safer than decoding the
-                    # virt-v2v output.
                     state['disks'].append({
-                        'path': d.encode('utf-8'),
+                        'path': d,
                         'progress': 0})
                 logging.debug('Internal disk list: %r', state['disks'])
                 state['disk_count'] = len(data['source_disks'])
@@ -2152,9 +2201,9 @@ def main():
 
             # Send some useful info on stdout in JSON
             print(json.dumps({
-                'v2v_log': v2v_log,
+                'v2v_log': state.v2v_log,
                 'wrapper_log': wrapper_log,
-                'state_file': state.get_filename(),
+                'state_file': state.state_file,
                 'throttling_file': throttling_file,
             }))
 
@@ -2174,10 +2223,11 @@ def main():
             agent_pid = None
             agent_sock = None
             if data['transport_method'] == 'ssh':
-                agent_pid, agent_sock = spawn_ssh_agent(data)
+                agent_pid, agent_sock = spawn_ssh_agent(
+                    data, host.get_uid(), host.get_gid())
                 if agent_pid is None:
                     raise RuntimeError('Failed to start ssh-agent')
-            wrapper(host, data, state, v2v_log, virt_v2v_caps, agent_sock)
+            wrapper(host, data, virt_v2v_caps, agent_sock)
             if agent_pid is not None:
                 os.kill(agent_pid, signal.SIGTERM)
             if not state.get('failed', False):
